// file : input.cpp

// using raw input
// https://msdn.microsoft.com/en-us/library/windows/desktop/ms645546(v=vs.85).aspx

// designing a robust input handling system for games
// http://www.gamedev.net/blog/355/entry-2250186-designing-a-robust-input-handling-system-for-games


#include "../../stdafx.h"
#include "input.h"

using namespace Core::Input;


cInput::cInput()
{
	mouseSensitivity = 0.02f;
}


bool cInput::registerDevice(HWND _hTarget, USHORT _usage)
{
	RAWINPUTDEVICE rid;
	rid.usUsagePage = 0x01;
	rid.usUsage = _usage;
	rid.dwFlags = RIDEV_DEVNOTIFY | RIDEV_INPUTSINK; // ignore legacy messages???
	rid.hwndTarget = _hTarget;

	if (RegisterRawInputDevices(&rid, 1, sizeof(RAWINPUTDEVICE)) == false)
	{
		Log("Error", "Failed to register raw input device (usage : %d)", _usage);
		return false;
	}

	return true;
}


void cInput::unregisterDevice(USHORT _usage)
{
	RAWINPUTDEVICE rid;
	rid.usUsagePage = 0x01;
	rid.usUsage = _usage;
	rid.dwFlags = RIDEV_REMOVE;
	rid.hwndTarget = NULL;
	RegisterRawInputDevices(&rid, 1, sizeof(RAWINPUTDEVICE));
}


void cInput::remapKey(short _key)
{
	if (actionPrevious[actionMap.map[_key]] == eKeyState::clicked)
	{
		action[actionMap.map[_key]] = eKeyState::held;
	}

	if (actionPrevious[actionMap.map[_key]] == eKeyState::released)
	{
		actionPrevious[actionMap.map[_key]] = eKeyState::clicked;
		action[actionMap.map[_key]] = eKeyState::clicked;
	}
}



void cInput::update(float _deltaTime)
{
	if (focusWnd->keysActive.size() > 0)
	{
		for (auto item : focusWnd->keysActive)
		{   // handle presses (click and hold)
			remapKey(item);
		}
	}

	while (focusWnd->keysDeactivate.size() > 0)
	{   // handle release
		int fromMap = actionMap.map[focusWnd->keysDeactivate.back()];
		action[fromMap] = actionPrevious[fromMap] = Input::eKeyState::released;
		focusWnd->keysDeactivate.pop_back();
	}

	// process mouse movement
	if (focusWnd->mouseX)
		mouseXoffset += focusWnd->mouseX / _deltaTime * mouseSensitivity;
	if (focusWnd->mouseY)
		mouseYoffset -= focusWnd->mouseY / _deltaTime * mouseSensitivity;
	focusWnd->mouseX = focusWnd->mouseY = 0;
}

