﻿// file : game.cpp

#include "../../stdafx.h"
#include "game.h"
#include "../stateManager.h"

using namespace Core::States;
using namespace Physics;


void cGame::initialize()
{
	scoreFont.initialize("Expansiva bold.otf", 24, clientSize);
	debugFont.initialize("verdana.ttf", 9, clientSize);
	
	tex_white = Graphics::cTexture("white.png");
	
	paddle_player = new Game::cPaddle(glm::vec2(-0.95f, 0.0f));
	paddle_ai     = new Game::cPaddle(glm::vec2( 0.95f, 0.0f));

	// center of field, get ready graphic
	float vertices_ready[] = {
		-0.5f,  0.5f, 0.0f, 0.0f, 1.0f,
		-0.5f, -0.5f, 0.0f, 0.0f, 0.0f,
		 0.5f,  0.5f, 0.0f, 1.0f, 1.0f,
		 0.5f, -0.5f, 0.0f, 1.0f, 0.0f,
	};
	Graphics::createArrayObject(vertices_ready, 4, vao_ready, vbo_ready);
	readyScalar = -1.0f;
	xFade = 0.0f;

	// center of field divider
	float vertices_divider[] = {
		-0.005f,  0.03f, 0.0f, 0.0f, 1.0f,
		-0.005f, -0.03f, 0.0f, 0.0f, 0.0f,
		 0.005f,  0.03f, 0.0f, 1.0f, 1.0f,
		 0.005f, -0.03f, 0.0f, 1.0f, 0.0f,
	};
	Graphics::createArrayObject(vertices_divider, 4, vao_floorDivider, vbo_floorDivider);

	reset();
}


void cGame::shutdown()
{
	safe_delete(paddle_player);
	safe_delete(paddle_ai);
}


void cGame::pause()
{}

void cGame::resume()
{}


void cGame::reset()
{
	winGameScore = 4;
	scorePlayer  = 0;
	scoreAI      = 0;
	gameTime     = 0.0f;
}


void cGame::handleEvents()
{}


char* cGame::getTimeString()
{
	static char buffer[16];

	// todo : put a cap on game time to avoid overflow
	int minutes = int(gameTime / 60);
	int hours = int(minutes / 60);
	minutes = minutes % 60;

	float seconds = gameTime - (hours * 3600) - (minutes * 60);
	sprintf_s(buffer, "%d::%d:%.1f", hours, minutes, seconds);
	
	return buffer;
}


char* cGame::getWinnerName()
{
	if (scorePlayer == winGameScore)
		return "Player";
	else
		return "Computer";
}


void cGame::drawText()
{
	scoreFont.enableState(pCore->useShader("text"));
	
	char buffer[16];
	scoreFont.setCaretPos(250, 580);
	_itoa_s(scorePlayer, buffer, 10);
	scoreFont.draw(buffer);

	scoreFont.setCaretPos(341, 580);
	scoreFont.draw(getTimeString());

	scoreFont.setCaretPos(530, 580);
	_itoa_s(scoreAI, buffer, 10);
	scoreFont.draw(buffer);

	scoreFont.disableState();

	// ------------------------------------
	/*
	char debugBuffer[256];
	debugFont.enableState(pCore->useShader("text"));
	debugFont.setCaretPos(10, 570);
	glm::vec2 bp = ball.getPosition();
	glm::vec2 pp = paddle_ai->getPosition();
	glm::vec2 ap = ai.getPosOffset();

	sprintf_s(debugBuffer, "Ball (%.2f, %.2f)\nAI-P (%.2f, %.2f)\nAI (%.2f, %.2f)", bp.x, bp.y, pp.x, pp.y, ap.x, ap.y);
	debugFont.draw(debugBuffer);
	debugFont.disableState();
	*/
}


void cGame::drawFloorDivider()
{
	tex_white.use();
	glBindVertexArray(vao_floorDivider);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glBindVertexArray(0);
}


void cGame::drawReadyServe()
{
	if (readyScalar < 0.0f)
		return;

	GLuint id = pCore->useShader("ball");
	GLuint offsetLoc = glGetUniformLocation(id, "offset");
	GLuint coreColorLoc = glGetUniformLocation(id, "coreColor");
	GLuint bumpColorLoc = glGetUniformLocation(id, "bumpColor");
	GLuint colorScaleLoc = glGetUniformLocation(id, "colorScalar");
	GLuint xFadeLoc = glGetUniformLocation(id, "xFade");

	glUniform2fv(offsetLoc, 1, glm::value_ptr(glm::vec3(0.0f)));
	glUniform4fv(coreColorLoc, 1, glm::value_ptr(glm::vec4(1.0f, 0.5f, 0.0f, 0.0f)));
	glUniform4fv(bumpColorLoc, 1, glm::value_ptr(glm::vec4(0.0f, 1.0f, 0.0f, 1.0f)));
	glUniform1f(colorScaleLoc, readyScalar);
	glUniform1f(xFadeLoc, xFade);

	glBindVertexArray(vao_ready);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glBindVertexArray(0);
}


void cGame::update(float _deltaTime)
{
	gameTime += _deltaTime;
	if (scorePlayer == winGameScore || scoreAI == winGameScore)
	{
		static_cast<Core::cStateManager*>(pManager)->handleEvents(Core::eStateAction::win);
	}

	// ball <---> wall logic
	int result = ball.update(_deltaTime);
	if (result > 0)
	{   // player scored
		scorePlayer++;
		readyScalar = 0.0f;
		ball.reset(1.0f);
		xFade = 1.0f;
	}
	if (result < 0)
	{ // ai scored
		scoreAI++;
		readyScalar = 0.0f;
		ball.reset(-1.0f);
		xFade = -1.0f;
	}


	if (readyScalar >= 0.0f)
	{  // gfx : center half bloom on new ball serve
		readyScalar += _deltaTime * 0.7f;
		if (readyScalar > 1.0f)
		{
			readyScalar = -1.0f;
		}
	}


	// player paddle 
	static float playerRot = 0.0f;
	static float lastRotOffset = 0.0f;
	switch (pCore->input.action[Core::Input::eAction::modifier])
	{
	case Core::Input::eKeyState::clicked:
		lastRotOffset = pCore->input.mouseXoffset;
		break;
	case Core::Input::eKeyState::held:
		paddle_player->update(lastRotOffset, pCore->input.mouseYoffset, playerRot);
		playerRot = (pCore->input.mouseXoffset - lastRotOffset) * 0.002f;
		break;
	case Core::Input::eKeyState::released:
		if (playerRot != 0.0f)
		{
			::SetCursorPos((int)lastRotOffset, (int)pCore->input.mouseYoffset);
			playerRot = 0.0f;
		}
		else
			paddle_player->update(pCore->input.mouseXoffset, pCore->input.mouseYoffset, playerRot);

		break;
	}

	//paddle_player->update(pCore->input.mouseXoffset, pCore->input.mouseYoffset, playerRot);

	// ball <---> player paddle logic
	sCircle c(ball.getPosition(), 0.05f);
	sRectangle r(paddle_player->getPosition(), 0.02f, 0.2f);
	if(intersectCircleRectangle(c, r))
	{
		ball.respond(paddle_player->getRotation(), -1);
	}

	// ai paddle
	ai.update(_deltaTime, ball.getPosition(), ball.getVelocity());
	glm::vec2 offset = ai.getPosOffset() * glm::vec2(0.0f, 8.0f);
	paddle_ai->update(offset.x, offset.y, 0.0f);
	sRectangle AIr(paddle_ai->getPosition() + glm::vec2(0.06f, 0.0f), 0.02f, 0.2f);
	if (intersectCircleRectangle(c, AIr))
	{
		ball.respond(paddle_ai->getRotation(), 1);
	}
}


void cGame::draw()
{
	glDisable(GL_DEPTH_TEST);

	GLuint id = pCore->useShader("basic");
	GLuint offsetLoc = glGetUniformLocation(id, "offset");
	glm::vec2 offset = glm::vec2(0.0f, -1.05f);
	for (int i = 0; i < 19; ++i)
	{
		offset.y += 0.1f;
		glUniform2fv(offsetLoc, 1, glm::value_ptr(offset));
		drawFloorDivider();
	}

	paddle_player->draw();
	paddle_ai->draw();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	drawReadyServe();
	ball.draw();
	
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
}
