// file : gameState.h

#ifndef _engine_core_states_game_h_
#define _engine_core_states_game_h_

#include "base_state.h"
#include "../../Graphics/Text/font.h"
#include "../../Physics/physics.h"
#include "../../Game/ball.h"
#include "../../Game/paddle.h"
#include "../../AI/ai.h"

namespace Core 
{
	namespace States 
	{

		class cGame : public cBase_state
		{

		public:
			cGame(void* _manager, glm::vec2 _clientSize) : cBase_state(_manager, "game_state", _clientSize) { initialize(); }

			void initialize();
			void shutdown();

			void pause();
			void resume();
			void reset();

			char* getTimeString();
			char* getWinnerName();

			void handleEvents();
			void update(float _deltaTime);
			void drawText();
			void draw();

			void drawFloorDivider();
			void drawReadyServe();

		private:
			Graphics::Text::cFont scoreFont;
			Graphics::Text::cFont debugFont;
			int scorePlayer;
			int scoreAI;
			float gameTime;
			int winGameScore;


			Graphics::cTexture tex_white;

			// game content goes here
			Game::cBall    ball;
			Game::cPaddle* paddle_player;
			Game::cPaddle* paddle_ai;

			AI::cAI ai;

			// background graphics -----------------------------
			GLuint vao_ready;
			GLuint vbo_ready;
			float readyScalar;
			float xFade;

			GLuint vao_floorDivider;
			GLuint vbo_floorDivider;

		};
	}
}

#endif//_engine_core_states_game_h_
