// file : intro.cpp

// http://nehe.gamedev.net/tutorial/playing_avi_files_in_opengl/23001/          // the hook
// http://nehe.gamedev.net/tutorial/radial_blur__rendering_to_a_texture/18004/  // and the effect (todo)


#include "../../stdafx.h"
#include "intro.h"
#include "../stateManager.h"

#include <Vfw.h> // microsoft video for windows
#pragma comment(lib, "vfw32.lib")

using namespace Core::States;


float upTime = 0.0f;

PAVISTREAM pavi;       // stream handle
PGETFRAME pgf;         // GetFrame object handle
long lastFrame;        // last frame of the stream
int width;             // video width
int height;            // video height
int mpf;               // milliseconds per frame

HDRAWDIB hdd;          // dib handle
HBITMAP hBitmap;       // device dependent bitmap handle
HDC hdc;               // device context handle


void cIntro::initialize()
{
	std::wstring filePath = L"Assets/Movies/goliathForgeDisolve_8x6.avi";

	AVIFileInit();  // open AVIFile Library
	if (AVIStreamOpenFromFile(&pavi, filePath.c_str(), streamtypeVIDEO, 0, OF_READ, nullptr) != 0)
	{
		Log("Warning", "Failed to open video file : %ws", filePath.c_str());
		return;
	}

	AVISTREAMINFO psi;
	AVIStreamInfo(pavi, &psi, sizeof(psi)); // avi stream info
	width = psi.rcFrame.right - psi.rcFrame.left;
	height = psi.rcFrame.bottom - psi.rcFrame.top;
	lastFrame = AVIStreamLength(pavi);
	mpf = AVIStreamSampleToTime(pavi, lastFrame) / lastFrame;

	unsigned char* data;
	BITMAPINFOHEADER bmi;

	hdc = CreateCompatibleDC(0);
	bmi.biSize = sizeof(BITMAPINFOHEADER);
	bmi.biPlanes = 1;
	bmi.biBitCount = 24;
	bmi.biWidth = width;
	bmi.biHeight = height;
	bmi.biCompression = BI_RGB;
	hBitmap = CreateDIBSection(hdc, (BITMAPINFO*)(&bmi), DIB_RGB_COLORS, (void**)(&data), nullptr, 0);
	SelectObject(hdc, hBitmap); // dib created to directly draw to

	pgf = AVIStreamGetFrameOpen(pavi, nullptr);
	if (pgf == nullptr)
	{
		Log("Warning", "Failed to open first frame of video");
		return;
	}

	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_BGR, GL_UNSIGNED_BYTE, data); // flipped rgb --> bgr
	glBindTexture(GL_TEXTURE_2D, 0);
}


void cIntro::shutdown()
{
	DeleteObject(hBitmap);
	DrawDibClose(hdd);
	AVIStreamGetFrameClose(pgf);
	AVIStreamRelease(pavi);
	AVIFileExit();
}


void cIntro::pause()
{}

void cIntro::resume()
{}

void cIntro::handleEvents()
{}


void cIntro::update(float _deltaTime)
{
	upTime += _deltaTime;
	int frame = (int)((upTime * 1000) / mpf);
	if (frame >= lastFrame)
	{
		// avi file completed last frame, change state to menu
		static_cast<Core::cStateManager*>(pManager)->popState();
		return;
	}

	LPBITMAPINFOHEADER lpbi;
	lpbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pgf, frame);
	char* pData = (char*)lpbi + lpbi->biSize + lpbi->biClrUsed * sizeof(RGBQUAD);
	DrawDibDraw(hdd, hdc, 0, 0, width, height, lpbi, pData, 0, 0, width, height, 0);

	glBindTexture(GL_TEXTURE_2D, texID);
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, width, height, GL_BGR, GL_UNSIGNED_BYTE, pData);
	glBindTexture(GL_TEXTURE_2D, 0);
}


void cIntro::drawText()
{
}


void cIntro::draw()
{
	GLuint id = pCore->useShader("basic");
	GLint offsetLoc = glGetUniformLocation(id, "offset");
	glm::vec2 offset(0.0f, 0.0f);
	GLint orientationLoc = glGetUniformLocation(id, "orientation");
	glm::mat4 orientation(1.0f);

	static std::once_flag init_vao_onceFlag;
	std::call_once(init_vao_onceFlag, [this]
	{   // fullscreen quad
		float vertices[] = {
			-1.0f,  1.0f, 0.0f, 0.0f, 1.0f,
			-1.0f, -1.0f, 0.0f, 0.0f, 0.0f,
			 1.0f,  1.0f, 0.0f, 1.0f, 1.0f,
			 1.0f, -1.0f, 0.0f, 1.0f, 0.0f,
		};

		glGenVertexArrays(1, &vao);
		glGenBuffers(1, &vbo);
		glBindVertexArray(vao);
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glBufferData(GL_ARRAY_BUFFER, sizeof(vertices), vertices, GL_STATIC_DRAW);
		glEnableVertexAttribArray(0);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 5 * sizeof(float), (void*)0);
		glEnableVertexAttribArray(1);
		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 5 * sizeof(float), (void*)(3 * sizeof(float)));
		glBindVertexArray(0);
	});

	glBindTexture(GL_TEXTURE_2D, texID);

	glBindVertexArray(vao);
	glUniform2fv(offsetLoc, 1, glm::value_ptr(offset));
	glUniformMatrix4fv(orientationLoc, 1, GL_FALSE, glm::value_ptr(orientation));
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glBindVertexArray(0);

	glBindTexture(GL_TEXTURE_2D, 0);
}
