// file : shaderManager.h

#ifndef _engine_core_shader_manager_h_
#define _engine_core_shader_manager_h_

#include <unordered_map>
#include "../Graphics/base_shader.h"


namespace Core
{

	class cShaderManager
	{

	public:

		~cShaderManager()
		{
			shader_map.clear();
		}


		GLuint useProgram(std::string _name)
		{
			std::unordered_map<std::string, GLuint>::const_iterator item = shader_map.find(_name);
			if (item == shader_map.end())
			{
				Log("Warning", "Shader not found in map");
			}
			else
			{
				GLuint id = item->second;
				glUseProgram(id);
				return id;
			}
			return 0;
		}


		bool insertProgram(std::string _name)
		{
			for (const auto pair : shader_map)
			{
				if (strcmp(pair.first.c_str(), _name.c_str()) == 0)
				{
					return false;
				}
			}

			GLuint programID = Graphics::cBase_shader(_name.c_str()).program;
			if (programID > 0)
			{
				shader_map[_name] = programID;
				return true;
			}
			return false;
		}


	private:

		std::unordered_map<std::string, GLuint> shader_map;

	};

}

#endif//_engine_core_shader_manager_h_
