// file : stateManager.cpp

#include "../stdafx.h"
#include "stateManager.h"
#include "States/intro.h"
#include "States/game.h"
#include "States/menu.h"
#include "States/scores.h"
#include "States/win.h"

using namespace Core;

std::shared_ptr<States::cIntro>  introState;
std::shared_ptr<States::cMenu>   menuState;
std::shared_ptr<States::cGame>   gameState;
std::shared_ptr<States::cScores> scoreState;
std::shared_ptr<States::cWin>    winState;

void cStateManager::initialize(glm::vec2 clientSize)
{
	introState = std::make_shared<States::cIntro> (this, clientSize);
	menuState  = std::make_shared<States::cMenu>  (this, clientSize);
	gameState  = std::make_shared<States::cGame>  (this, clientSize);
	scoreState = std::make_shared<States::cScores>(this, clientSize);
	winState   = std::make_shared<States::cWin>   (this, clientSize);

	states.push_back(menuState);

#ifdef _RELEASE
	states.push_back(introState);
#endif
}


void cStateManager::shutdown()
{
	introState->shutdown();
	menuState->shutdown();
	gameState->shutdown();
	scoreState->shutdown();
	winState->shutdown();
}


void cStateManager::changeState(States::cBase_state* _state)
{}


void cStateManager::pushState(States::cBase_state* _state)
{}


void cStateManager::popState()
{
	//states.back()->shutdown();
	states.pop_back();
}



void cStateManager::handleEvents(eStateAction _action)
{
	switch (_action)
	{

	case eStateAction::pause:
		if (states.back()->name == "game_state")
			states.pop_back();
		break;

	case eStateAction::play:
		if (states.back()->name == "menu_state")
		{
			RECT rect;
			::GetWindowRect(pCore->input.getFocusWindowHandle(), &rect);
			::ClipCursor(&rect);
			::ShowCursor(false);
			pCore->input.mouseXoffset = pCore->input.mouseYoffset = 0.0f;
			states.push_back(gameState);
		}
		break;

	case eStateAction::highScore:
		if (states.back()->name == "menu_state")
		{
			states.push_back(scoreState);
		}
		break;

	case eStateAction::win:
		if (states.back()->name == "game_state")
		{
			winState->winnerName = gameState->getWinnerName();
			winState->winningTime = gameState->getTimeString();

			states.push_back(winState);
		}
		break;

	case eStateAction::quit:
		if (states.back()->name == "menu_state")
			PostQuitMessage(0);

		if (states.back()->name == "game_state")
		{
			::ShowCursor(true);
			::ClipCursor(nullptr);
			states.pop_back();
		}

		if (states.back()->name == "score_state")
			states.pop_back();

		if (states.back()->name == "win_state")
		{
			
			// todo : record score
			
			gameState->reset();
			states.pop_back();
		}

		break;

	} // end switch(action)
}


void cStateManager::update(float deltaTime)
{
	states.back()->update(deltaTime);
}


void cStateManager::drawText()
{
	states.back()->drawText();
}


void cStateManager::draw()
{
	states.back()->draw();
}
