// file : timer.cpp

#include "../stdafx.h"
#include "timer.h"

using namespace Core;

cTimer::cTimer() : secondsPerCount(0.0),
                   deltaTime(-1.0),
                   baseTime(0),
                   pausedTime(0),
                   prevTime(0),
                   thisTime(0),
                   stopped(false)
{
	__int64 countsPerSecond;
	QueryPerformanceFrequency((LARGE_INTEGER*)&countsPerSecond);
	secondsPerCount = 1.0 / (double)countsPerSecond;
}


float cTimer::getTotalTime() const
{
	if (stopped)
	{
		return (float)(((stopTime - pausedTime) - baseTime) * secondsPerCount);
	}
	else
	{
		return (float)(((thisTime - pausedTime) - baseTime) * secondsPerCount);
	}
}


float cTimer::getDeltaTime() const
{
	return (float)deltaTime;
}


void cTimer::reset()
{
	__int64 currentTime;
	QueryPerformanceCounter((LARGE_INTEGER*)&currentTime);

	baseTime = currentTime;
	prevTime = currentTime;
	stopTime = 0;
	stopped = false;
}


void cTimer::start()
{
	__int64 startTime;
	QueryPerformanceCounter((LARGE_INTEGER*)&startTime);

	if (stopped)
	{
		pausedTime += (startTime - stopTime);
		stopTime = 0;
		stopped = false;
	}
}


void cTimer::stop()
{
	if (!stopped)
	{
		_int64 currentTime;
		QueryPerformanceCounter((LARGE_INTEGER*)&currentTime);

		stopTime = currentTime;
		stopped = true;
	}
}


float cTimer::tick()
{
	if (stopped)
	{
		deltaTime = 0;
		return 0.0f;
	}

	QueryPerformanceCounter((LARGE_INTEGER*)&thisTime);

	deltaTime = (thisTime - prevTime) * secondsPerCount;
	prevTime = thisTime;

	if (deltaTime < 0.0)
	{
		deltaTime = 0.0;
	}

	return (float)deltaTime;
}
