// file : ball.cpp

#include "../stdafx.h"
#include "ball.h"

using namespace Game;

cBall::cBall()
{
	coreColor = glm::vec4(1.0f, 1.0f, 1.0f, 1.0f);
	bumpColor = glm::vec4(1.0f, 1.0f, 0.0f, 1.0f);
	colorScalar = 1.0f;

	reset(1.0f);

	float vertices[] = {
		-0.05f,  0.07f, 0.0f, 0.0f, 1.0f,
		-0.05f, -0.07f, 0.0f, 0.0f, 0.0f,
		 0.05f,  0.07f, 0.0f, 1.0f, 1.0f,
		 0.05f, -0.07f, 0.0f, 1.0f, 0.0f,
	};
	Graphics::createArrayObject(vertices, 4, vao, vbo);
}


void cBall::reset(float _sign)
{
	position = glm::vec2(0.0f, 0.0f);
	velocity = glm::vec2(0.7f * _sign, 0.9f);
}


int cBall::update(float _deltaTime)
{
	int result = 0;

	position += velocity * _deltaTime;
	// keep ball in screen bounds
	// set to bump color if wall collision
	if (position.x > 1.0f)
	{   // ball hit the back wall on the AI side (right)
		position.x = 1.0f;
		velocity.x = -abs(velocity.x);
		colorScalar = 0.0f;
		result = 1; 
	}
	else if (position.x < -1.0f)
	{   // ball hit the back wall on the player side (left)
		position.x = -1.0f;
		velocity.x = abs(velocity.x);
		colorScalar = 0.0f;
		result = -1;
	}

	else if (position.y > 1.0f)
	{  // ball hit the top wall
		position.y = 1.0f;
		velocity.y = -abs(velocity.y);
		colorScalar = 0.0f;
	}
	else if (position.y < -1.0f)
	{   // ball hit the bottom wall
		position.y = -1.0f;
		velocity.y = abs(velocity.y);
		colorScalar = 0.0f;
	}

	colorScalar += _deltaTime;
	if (colorScalar > 1.0f)
		colorScalar = 1.0f;

	// hack : try to save a locked ball
	if (abs(velocity.x) < 0.1)
		velocity.x *= 5.0f + 0.01f;

	return result;
}


void cBall::respond(float _rotation, float _fieldSign)
{
	//
	//
	//       /   The solution for ball handling I'm going for here will be a rotating paddle.
	//  _   /    
	// | | /     
	// | |/      
	// | |\      
	//  ~  \     
	//      \    
	//           
	
	if (velocity.x * _fieldSign > 0.0f)
	{   // only react if the ball is moving towards the paddle (eliminates paddle slide)
		glm::vec4 collisionNormal = glm::eulerAngleZ(_rotation) * glm::vec4(1.0f, 0.0f, 0.0f, 0.0f);

		float speed = glm::length(velocity);
		glm::vec2 reflection = glm::reflect(glm::normalize(velocity), glm::vec2(collisionNormal.x, collisionNormal.y));

		velocity.x = reflection.x * speed;
		velocity.y = reflection.y * speed;
	}
}


void cBall::draw()
{ 
	GLuint id = pCore->useShader("ball");
	GLuint offsetLoc = glGetUniformLocation(id, "offset");
	GLuint coreColorLoc = glGetUniformLocation(id, "coreColor");
	GLuint bumpColorLoc = glGetUniformLocation(id, "bumpColor");
	GLuint colorScaleLoc = glGetUniformLocation(id, "colorScalar");
	GLuint xFadeLoc = glGetUniformLocation(id, "xFade");


	//static std::once_flag init_vao_onceFlag;
	//std::call_once(init_vao_onceFlag, [this]
	//{ /* put once stuff here*/ });
	

	glBindVertexArray(vao);
	glUniform2fv(offsetLoc, 1, glm::value_ptr(position));

	glUniform4fv(coreColorLoc, 1, glm::value_ptr(coreColor));
	glUniform4fv(bumpColorLoc, 1, glm::value_ptr(bumpColor));
	glUniform1f(colorScaleLoc, colorScalar);
	glUniform1f(xFadeLoc, 0.0f);

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glBindVertexArray(0);
}
