// file : paddle.cpp

#include "../stdafx.h"
#include "paddle.h"

using namespace Game;

cPaddle::cPaddle(glm::vec2 _position)
{
	position = _position;
	rotation = 0.0f;

	float vertices[] = {
		-0.01f,  0.1f, 0.0f, 0.0f, 1.0f,
		-0.01f, -0.1f, 0.0f, 0.0f, 0.0f,
		 0.01f,  0.1f, 0.0f, 1.0f, 1.0f,
		 0.01f, -0.1f, 0.0f, 1.0f, 0.0f,
	};
	Graphics::createArrayObject(vertices, 4, vao, vbo);
}


void cPaddle::update(float _posX, float _posY, float _rotation)
{
	// _mouseX and _mouseY is not a delta, but is clamped 
	// Fine for vertical movement as as I'm drawing from negative one to positive one.
	// Perhaps better to set ortho projection from 0 to screen size in the core and use _mouseY directly
	// Minor problem : wild user mouse movement into outside dead zone...user needs to hunt for that sweet spot again to regain control.
	// works for vertical movement because the system cursor is clipped to the window during play mode.
	// possible fix would be to clip the system mouse horizontal to a narrow column that fits the small rotation range

	position.y = _posY / 600.0f; // paddle vertical movement (scale to -1 ---> 1)
	position.y = glm::clamp(position.y, -0.9f, 0.9f);

	if (position.x < 0.0f)
	{
		position.x = _posX / 600.0f;
		position.x = glm::clamp(position.x, -0.9f, -0.3f);
	}
	else
	{
		// the ai side...
		;
	}
	
	_rotation = glm::clamp(_rotation, -0.2f, 0.2f);
	matOrientation = glm::eulerAngleZ(_rotation);
}


void cPaddle::draw()
{
	GLuint id = pCore->useShader("basic");
	glUniformMatrix4fv(glGetUniformLocation(id, "orientation"), 1, GL_FALSE, glm::value_ptr(matOrientation));
	glUniform2fv(glGetUniformLocation(id, "offset"), 1, glm::value_ptr(position));
	
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glBindVertexArray(0);
}
