// file : base_shader.h

#ifndef _engine_graphics_base_shader_h_
#define _engine_graphics_base_shader_h_

#include <string>
#include <fstream>
#include <sstream>
#include <iostream>


namespace Graphics
{

	class cBase_shader
	{
	public:

		GLuint program;


		/// cBase_shader
		///
		/// constructior initialization
		cBase_shader(const GLchar* _name)
		{
			std::string vsPath("Assets/Shaders/"); vsPath += _name; vsPath += ".vs";
			std::string fsPath("Assets/Shaders/"); fsPath += _name; fsPath += ".fs";
			std::ifstream vsFile;
			std::ifstream fsFile;
			std::stringstream vsStream;
			std::stringstream fsStream;
			// enable ifstream object exceptions
			vsFile.exceptions(std::ifstream::badbit);
			fsFile.exceptions(std::ifstream::badbit);

			// do disk read
			std::string vsSource;
			std::string fsSource;
			try
			{
				vsFile.open(vsPath);
				fsFile.open(fsPath);
				vsStream << vsFile.rdbuf();
				fsStream << fsFile.rdbuf();
				vsFile.close();
				fsFile.close();

				vsSource = vsStream.str();
				fsSource = fsStream.str();

				if (vsSource == "" || fsSource == "")
				{
					Log("WARNING", "%s shader stream returned empty string", _name);
				}
			}
			catch (std::ifstream::failure e)
			{
				Log("ERROR", "%s shader was (not) sucessfully read from disk", _name);
			}

			// create the shader program
			const GLchar* vsCode = vsSource.c_str();
			const GLchar* fsCode = fsSource.c_str();
			GLint success;
			GLchar infoLog[512];

			GLuint vs = glCreateShader(GL_VERTEX_SHADER);
			glShaderSource(vs, 1, &vsCode, nullptr);
			glCompileShader(vs);
			glGetShaderiv(vs, GL_COMPILE_STATUS, &success);
			if (!success)
			{
				glGetShaderInfoLog(vs, 512, nullptr, infoLog);
				Log("ERROR", "%s vertex shader compilation failed\n%s", _name, infoLog);
			}

			GLuint fs = glCreateShader(GL_FRAGMENT_SHADER);
			glShaderSource(fs, 1, &fsCode, nullptr);
			glCompileShader(fs);
			glGetShaderiv(fs, GL_COMPILE_STATUS, &success);
			if (!success)
			{
				glGetShaderInfoLog(fs, 512, nullptr, infoLog);
				Log("ERROR", "%s fragment shader compilation failed\n%s", _name, infoLog);
			}

			program = glCreateProgram();
			glAttachShader(program, vs);
			glAttachShader(program, fs);
			glLinkProgram(program);
			glGetProgramiv(program, GL_LINK_STATUS, &success);
			if (!success)
			{
				glGetProgramInfoLog(program, 512, nullptr, infoLog);
				Log("ERROR", "%s shader program link failed\n%s", _name, infoLog);
			}

			glDeleteShader(vs);
			glDeleteShader(fs);
		}

		/*
		// uniform utilities
		void setBool(const std::string &_name, bool _value) const
		{
		glUniform1i(glGetUniformLocation(program, _name.c_str()), (int)_value);
		}

		void setInt(const std::string &_name, int _value) const
		{
		glUniform1i(glGetUniformLocation(program, _name.c_str()), _value);
		}

		void setFloat(const std::string &_name, float _value) const
		{
		glUniform1f(glGetUniformLocation(program, _name.c_str()), _value);
		}
		*/
	};

}// end namespace Graphics

#endif//_engine_graphics_base_shader_h_
