// file : main.cpp

// https://msdn.microsoft.com/en-us/library/windows/desktop/ms633559(v=vs.85).aspx WinMain entry point
// https://msdn.microsoft.com/en-us/library/windows/desktop/ms644928(v=vs.85).aspx#creating_loop Using Messages and Message Queues
// http://gamesfromwithin.com/the-care-and-feeding-of-pre-compiled-headers 

#include "stdafx.h"
#include "Core/timer.h"
#include "Core/stateManager.h"

int CALLBACK wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR cmdLine, int cmdShow)
{
	// warning level 4 supppression
	UNREFERENCED_PARAMETER(hInstance);
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(cmdLine);

#ifdef _DEBUG  // memory leak detection ------------------------------------------------------
	int debugFlag = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
	debugFlag |= _CRTDBG_ALLOC_MEM_DF;
	debugFlag |= _CRTDBG_LEAK_CHECK_DF;
	_CrtSetDbgFlag(debugFlag);

	int allocationNumber = -1; // set to allocation index value from debug output on leak detection
	_CrtSetBreakAlloc(allocationNumber);
#endif // ------------------------------------------------------------------------------------

	
	Core::cWindow window(800, 600);

	pCore->registerInput(&window);

	pCore->addShader("basic");
	pCore->addShader("text");
	pCore->addShader("ball");

	Core::cStateManager stateManager;
	stateManager.initialize(window.getSize());

	Core::cTimer timer;
	timer.reset();
	timer.start();


	ShowWindow(window.getHWnd(), cmdShow);

	// Main Message Loop
	MSG msg = { 0 };
	bool bQuit = false;
	while (!bQuit)
	{
		while (PeekMessage(&msg, nullptr, 0, 0, PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
				bQuit = true;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}


		// process frame ------------------------------------------------
		float deltaTime = timer.tick();
		pCore->input.update(deltaTime);

		// ---------------- game state manipulatin ----------------------
		if (pCore->input.action[Core::Input::eAction::escape] == Core::Input::eKeyState::clicked)
			stateManager.handleEvents(Core::eStateAction::quit);

		if (pCore->input.action[Core::Input::eAction::enter] == Core::Input::eKeyState::clicked)
			stateManager.handleEvents(Core::eStateAction::play);

		stateManager.update(deltaTime);


		// draw frame ----------------------------------------------------
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		stateManager.draw();
		stateManager.drawText();

		SwapBuffers(window.getDevice());
	}

	// clean up ----------------------------------------------------------
	pCore->input.unregisterDevice(0x06);
	pCore->input.unregisterDevice(0x02);
	stateManager.shutdown();
	pCore->shutdown();
	// window is destructor destroyed

	return msg.wParam;
}
